@@ -21,7 +21,7 @@ import itertools
 import collections
 
 __author__  = 'Miguel Turner'
-__version__ = '0.2.8'
+__version__ = '0.2.9'
 __license__ = 'MIT'
 
 __all__ = ['Config', 'IniFormat', 'ConfigError', 'Coercer', 'CoerceError']
@@ -63,7 +63,7 @@ class ConfigSection(collections.MutableMapping):
         else:
             # child
             self._root = parent._root
-            self._key = self._make_key(parent._key, name)
+            self._key = self._keystr(self._make_key(parent._key, name))
             parent._children[name] = self
         
         self._children = self._root._dict_type()
@@ -82,9 +82,7 @@ class ConfigSection(collections.MutableMapping):
     @property
     def key(self):
         """The section's key. Read-only."""
-        if self._key is None:
-            return None
-        return self._keystr(self._key)
+        return self._key
     
     @property
     def name(self):
@@ -106,6 +104,10 @@ class ConfigSection(collections.MutableMapping):
     def is_default(self):
         return (self._value is NoValue and self._default is not NoValue)
     
+    @property
+    def has_children(self):
+        return bool(self._children)
+    
     ## methods ##
     
     def sync(self, *sources, **kwargs):
@@ -277,23 +279,17 @@ class ConfigSection(collections.MutableMapping):
                 raise InvalidSectionError(key)
         return section
 
-    def sections(self, recurse=False, include_self=False):
+    def sections(self, recurse=False):
         """Returns the sections that are children to this section.
         
         If *recurse* is `True`, returns grandchildren as well.
-        If *include_self* is `True`, returns this section first.
         """
-        if include_self:
-            yield self
         for child in self._children.values():
             yield child
             if recurse:
                 for grand in child.sections(recurse):
                     yield grand
     
-    def has_children(self, key=None):
-        return self.section(key).has_children() if key else bool(self._children)
-    
     def reset(self, recurse=True):
         """Resets this section to it's default value, leaving it
         in the same state as after a call to :meth:`ConfigSection.init`.
@@ -521,9 +517,10 @@ class ConfigSection(collections.MutableMapping):
         return value if should_cache else NoValue
     
     def _dump(self, indent=2): # pragma: no cover
-        rootlen = len(self._key)
+        rootlen = len(self._make_key(self._key))
         for section in sorted(self.sections(recurse=True)):
-            spaces = ' ' * ((len(section._key) - rootlen) * indent - 1)
+            sectionlen = len(self._make_key(section._key))
+            spaces = ' ' * ((sectionlen - rootlen) * indent - 1)
             print(spaces, repr(section))
 
 class Config(ConfigSection):
@@ -575,7 +572,8 @@ class Config(ConfigSection):
     
     def _dump(self, indent=2): # pragma: no cover
         for section in self.sections(recurse=True):
-            spaces = ' ' * ((len(section._key) - 1) * indent)
+            sectionlen = len(self._make_key(section._key))
+            spaces = ' ' * ((sectionlen - 1) * indent)
             print(spaces, repr(section), sep='')
     
     def __repr__(self): # pragma: no cover
@@ -669,51 +667,52 @@ class IniFormat(Format):
     name = 'ini'
     delimeter = '='
     comment_char = ';'
-    _rx_section_header = re.compile('\[(.*)\]')
+    default_section = 'default'
+    _rx_section_header = re.compile('\[\s*(\S*)\s*\](\s*=\s*(\S*))?')
     
     def read(self, file):
         cfg = self.config
-        section_name = None
+        section_name = self.default_section
         comment = None
         lines = []
         values = cfg._dict_type()
         comments = {}
         
+        def flush_comment(lines, comment):
+            if comment:
+                lines.append(comment)
+        
         for i, orgline in enumerate(file, 1):
             line = orgline.strip()
             
             # blank line
             if not line:
-                if comment: # flush any comment
-                    lines.append(comment)
-                    comment = None
-                lines.append(Line(orgline))
+                comment = flush_comment(lines, comment)
                 continue
             
             # comment line
             if line.startswith(self.comment_char):
-                if comment: # flush any comment
-                    lines.append(comment)
-                    comment = None
+                flush_comment(lines, comment)
                 comment = Line(orgline, line.lstrip(self.comment_char).strip())
                 continue
             
             # section header
             match = self._rx_section_header.match(line)
             if match:
-                section_name = match.group(1)
-                values[section_name] = None
+                section_name, _, value = match.groups()
+                
+                # blank sections are set to default
+                if not section_name or section_name.lower() == self.default_section:
+                    section_name = self.default_section
+                
+                values[section_name] = value
                 comments[section_name] = comment.name if comment else None
                 comment = None
+                
                 lines.append(Line(orgline, section_name, issection=True))
                 continue
             
-            # ready for key/value, but no section has been read yet
-            if section_name is None:
-                self._read_error(file, i, line)
-                continue
-            
-            # get the value
+            # must be a value
             try:
                 key, value = line.split(self.delimeter, 1)
             except ValueError:
@@ -741,94 +740,80 @@ class IniFormat(Format):
         
         return lines
     
-    def write(self, file, lines=None):
-        stripbase = lambda k: cfg._keystr(cfg._make_key(k)[1:])
-        def write_section(section):
-            # strip the first section of the key
-            key = stripbase(section.key)
-            if key and section.comment:
-                file.write('{} {}\n'.format(self.comment_char, section.comment))
-            parts = filter(None, [key, self.delimeter, section.value(convert=False)])
-            file.write(' '.join(parts) + '\n')
-            section.dirty = False
+    def write_section(self, section, file, first=False):
+        if not first and section.parent is section.root:
+            file.write('\n')
         
+        if section.comment:
+            file.write('{} {}\n'.format(self.comment_char, section.comment))
+        
+        if section.parent is section.root:
+            # header section
+            if section.valid:
+                value = section.value(convert=False)
+                file.write('[{}] = {}\n'.format(section.name, value))
+            else:
+                file.write('[{}]\n'.format(section.name))
+        elif section.valid:
+            # value section
+            cfg = self.config
+            key = cfg._keystr(cfg._make_key(section.key)[1:])
+            value = section.value(convert=False)
+            
+            file.write('{} {} {}\n'.format(key, self.delimeter, value))
+        
+        section.dirty = False
+    
+    def write(self, file, lines=None):
         cfg = self.config
         
         # write back values in the order they were read
-        unseen = set(s.key for s in cfg.sections(recurse=True))
-        unseen_sections = set(s.key for s in cfg.sections())
-        section_name = None
+        seen = set()
+        header = None
         lines = lines or []
-        last_line = None
+        first = True
         for i, line in enumerate(lines):
             if line.issection:
-                if line.name not in unseen_sections:
+                if line.name in seen:
                     continue
                 
-                # if there is a previous section, write it's remaining values
-                if section_name:
-                    section = cfg.section(section_name)
-                    for sec in section.sections(recurse=True, include_self=True):
-                        if sec.valid and sec.key in unseen:
-                            write_section(sec)
-                            unseen.discard(sec.key)
+                # if there is a previous header section, write it's remaining values
+                if header:
+                    for sec in header.sections(recurse=True):
+                        if sec.key not in seen:
+                            self.write_section(sec, file)
+                            seen.add(sec.key)
                 
                 # write current section header
-                section = cfg.section(line.name)
-                if section.comment:
-                    file.write('{} {}\n'.format(self.comment_char, section.comment))
-                file.write('[{}]\n'.format(section.name))
-                
-                section_name = section.name
-                unseen_sections.discard(section_name)
+                header = cfg.section(line.name)
+                self.write_section(header, file, first)
+                seen.add(header.key)
+                first = False
             
             elif line.iskey:
-                if line.name not in unseen:
+                if line.name in seen:
                     continue
-                write_section(cfg.section(line.name))
-                unseen.discard(line.name)
+                self.write_section(cfg.section(line.name), file)
+                seen.add(line.name)
             
             else:
                 file.write(line.line)
-            
-            last_line = line
         
-        if last_line and not last_line.name:
-            file.seek(file.tell()-1)
-            file.truncate()
-        
-        # if there is a previous section, write it's remaining values
-        if section_name:
-            section = cfg.section(section_name)
-            for sec in section.sections(recurse=True, include_self=True):
-                if sec.valid and sec.key in unseen:
-                    write_section(sec)
-                    unseen.discard(sec.key)
+        # if there is a previous header section, write it's remaining values
+        if header:
+            for sec in header.sections(recurse=True):
+                if sec.key not in seen:
+                    self.write_section(sec, file)
+                    seen.add(sec.key)
         
         # write remaining values
-        for section in cfg.sections():
-            if section.key in unseen_sections:
-                # section header
-                if section.comment:
-                    file.write('{} {}\n'.format(self.comment_char, section.comment))
-                file.write('[{}]\n'.format(section.name))
-                
-                # section value
-                if section.valid:
-                    file.write('{} {}\n'.format(self.delimeter, section.value(convert=False)))
-                    section.dirty = False
-                
-                # subsection values
-                for sec in section.sections(recurse=True):
-                    if sec.valid and sec.key in unseen:
-                        write_section(sec)
-                        unseen.discard(sec.key)
-                
-                file.write('\n')
-        
-        if not lines:
-            file.seek(file.tell()-1)
-            file.truncate()
+        for section in cfg.sections(recurse=True):
+            if section.key in seen:
+                continue
+            
+            self.write_section(section, file, first)
+            seen.add(section.key)
+            first = False
 
 ## Config Errors ##
 
